package it.eng.spago.cms;

import java.util.Iterator;
import java.util.List;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.ISessionFactory;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import javax.jcr.LoginException;
import javax.jcr.NoSuchWorkspaceException;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;

/**
 * Implements methods to get the JSR 170 Session Object from the Exo CMS.
 */
public class ExoRepositoryFactoryImpl implements IRepositoryFactory {

	
	private static final String NAME_MODULE = "SPAGOCMS";
	
	/**
	 * JSR Repository
	 */
	protected Repository _repo = null;

	
	/**
	 * Construct of the class:  get the Exo repository using the exo portal services
	 */
	public ExoRepositoryFactoryImpl() {
		try{	
			String paramCont = "CONTENTCONFIGURATION.CONTENTREPOSITORY.PARAMETERS.CONTEXTNAME";
			ConfigSingleton config = ConfigSingleton.getInstance();
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryFactoryImpl::init:Spago configuration retrived");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryFactoryImpl::init:try to get the "+paramCont+
								" parameter from the cms configuration ");
			SourceBean paramContSB = (SourceBean)config.getAttribute(paramCont);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryFactoryImpl::init:cms sourcebean parameter context name retrived");
			String nameCont = (String)paramContSB.getAttribute("name");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryFactoryImpl::init:use "+nameCont+" context name");
			RootContainer rootCont = RootContainer.getInstance();
			List comps = rootCont.getComponentInstances();
			Iterator iter = comps.iterator();
			while(iter.hasNext()) {
				Object comp = iter.next();
				if(comp instanceof PortalContainer ) {
					PortalContainer port = (PortalContainer)comp;
					String name = port.getPortalContainerInfo().getContainerName();
					System.out.println(name);
				}
			}
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryFactoryImpl::init:Exo root container retrived");
			PortalContainer container = rootCont.getPortalContainer(nameCont); 
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryFactoryImpl::init:Exo "+nameCont+" portal container retrived");
			PortalContainer.setInstance(container);
			RepositoryService repositoryService = 
				(RepositoryService)container.getComponentInstanceOfType(RepositoryService.class);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryFactoryImpl:Exo repository service retrived");
			Repository repository = repositoryService.getRepository();
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"ExoRepositoryFactoryImpl:Exo jcr repository retrived");
			_repo = repository;
		} catch (RepositoryException e) {
			TracerSingleton.log(NAME_MODULE,
					            TracerSingleton.CRITICAL,
					            "ExoRepositoryFactoryImpl::init: cannot get the repository", e);
		} catch (RepositoryConfigurationException e) {
			TracerSingleton.log(NAME_MODULE,
		            TracerSingleton.CRITICAL,
		            "ExoRepositoryFactoryImpl::init: cannot get the repository", e);
		} catch (Exception e) {
			TracerSingleton.log(NAME_MODULE,
		            TracerSingleton.CRITICAL,
		            "ExoRepositoryFactoryImpl::init: cannot get the repository", e);
		}finally {	}
	}
	
	public Repository getRepository() {
		return _repo;
	}


}




