package it.eng.spago.cms;

import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import javax.jcr.Repository;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;


/**
 * Implements methods to get the JSR 170 Session Object from the Exo CMS.
 */
public class JndiRepositoryFactoryImpl implements IRepositoryFactory {

	
	private static final String NAME_MODULE = "SPAGOCMS";
	
	/**
	 * JSR Repository
	 */
	protected Repository _repo = null;

	
	/**
	 * Construct of the class:  get the jcr repository from a jndi tree
	 */
	public JndiRepositoryFactoryImpl() {
		try{	
			String paramContext = "CONTENTCONFIGURATION.CONTENTREPOSITORY.PARAMETERS.JNDICONTEXTNAME";
			String paramObject = "CONTENTCONFIGURATION.CONTENTREPOSITORY.PARAMETERS.JNDIOBJECTNAME";
			ConfigSingleton config = ConfigSingleton.getInstance();
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryFactoryImpl::init:Spago configuration retrived: " + config);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryFactoryImpl::init:try to get the "+paramContext+
								" parameter from the cms configuration ");
			SourceBean paramContextSB = (SourceBean)config.getAttribute(paramContext);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryFactoryImpl::init:cms sourcebean parameter jndi context " +
								"name retrived: " + paramContextSB);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					            "JndiRepositoryFactoryImpl::init:try to get the "+paramObject+
					            " parameter from the cms configuration ");
			SourceBean paramObjectSB = (SourceBean)config.getAttribute(paramObject);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryFactoryImpl::init:cms sourcebean parameter jndi object " +
								"name retrived: " + paramObjectSB);
			String nameContext = (String)paramContextSB.getAttribute("name");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryFactoryImpl::init:use "+nameContext+" context name");
			String nameObject = (String)paramObjectSB.getAttribute("name");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryFactoryImpl::init:use "+nameObject+" object name");
            InitialContext initCtx = new InitialContext();
            TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					            "JndiRepositoryFactoryImpl::init:jndi initial context created: " + initCtx);
            Context envCtx = (Context)initCtx.lookup(nameContext);
            TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
            					"JndiRepositoryFactoryImpl::init:jndi context "+nameContext+" retrived: " + envCtx);
            Object obj = envCtx.lookup(nameObject);
            TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryFactoryImpl::init:jndi object "+nameObject+" retrived: " + obj);
	        _repo = (Repository)obj;
	        
		} catch (NamingException ne) {
			TracerSingleton.log(NAME_MODULE, 
        			TracerSingleton.CRITICAL, 
        			"JndiRepositoryFactoryImpl::init: cannot get the repository", ne); 
		} catch (Exception e) { 
			TracerSingleton.log(NAME_MODULE, 
		            			TracerSingleton.CRITICAL, 
		            			"JndiRepositoryFactoryImpl::init: cannot get the repository", e); 
		} finally {	}
	}
	
	public Repository getRepository() {
		return _repo;
	}


}




